/*
 * File: minicontrol.c
 *
 * this file is part of the "minicontrol" project
 * Copyright (C) 2003 by Jean Philippe GUILLEMIN <jp.guillemin@free.fr>
 * license: This software is under GPL license
 * rev: 0.2  :))
 */



#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <getopt.h>
#include <sysexits.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkx.h>

#ifndef __CONFPATH
#define __CONFPATH "/usr/share/minicontrol"
#endif

#ifndef __CONFFILE
#define __CONFFILE "minicontrol.cf"
#endif

#define SHORT_STRING 64
#define MEDIUM_STRING 128
#define BIG_STRING 256
#define LONG_STRING 1024
#define EXTRALONG_STRING 4096




// Prototypes ************************************************************

GtkWidget *create_main_dockapp_icon_window
		(GtkWidget *main_window,
		unsigned int size,
		int main_argc,
		char *main_argv[]);

void launchonclick
		(GtkWidget *event_box, 
		GdkEventButton *event,
		gpointer data);

void launchdialog
		(GtkWidget *parent, 
		GdkEventButton *event,
		gpointer data);

int parse_from_config
		(u_int32_t *uint_value, 
		char *char_value, 
		char *filename, 
		char *param );

int usage(void);

int getclocktime(char *clockstring);

gint draw_label( gpointer data );

void exitwidget (GtkWidget *wgt, gpointer data);


// *******************************************************************

char	 	*configfile,
				*parameter,
				*char_param_value,
				*text_color,
				*text_size;
u_int32_t uint_param_value;



// Start main *******************************************************************

int main( int argc, char *argv[] ) {

	int i;
	GtkWidget *mainwin;
	GtkWidget *mainbox;
	GtkWidget *pixmap1,*pixmap2;
	GdkBitmap *mask;
	GtkWidget *pixmap_widget1,*pixmap_widget2;
	GtkWidget *clockbox, *box1, *box2;
	GtkWidget *watchlabel;
	GtkWidget *separator;
	GtkWidget *table;

	char	 	*wprefs_cmd, 
					*imagefile1, 
					*imagefile2;

	configfile = malloc(MEDIUM_STRING);
	parameter = malloc(MEDIUM_STRING);
	char_param_value = malloc(BIG_STRING); 

	
	wprefs_cmd = malloc(BIG_STRING);

	imagefile1 = malloc(BIG_STRING);
	imagefile2 = malloc(BIG_STRING);
	text_color = malloc(BIG_STRING);
	text_size = malloc(BIG_STRING);
		
	
// Here we parse command line args and configfile
	


	if (argc < 2){
		strncpy(configfile, __CONFPATH, MEDIUM_STRING);	
		strcat(configfile, "/");
		strcat(configfile, __CONFFILE);	
	}else{
		while ((i = getopt(argc, argv, "hc:p:")) != EOF){
				switch (i){
					case 'c': /* config file */
						strncpy(configfile, __CONFPATH, MEDIUM_STRING);	
						strcat(configfile, optarg);
						break;
					case 'p': /* custom config file */
						strcat(configfile, optarg);
						break;
					case 'h': usage(); exit (EXIT_SUCCESS);
					}
			}
		}
	//fprintf(stderr, "%s\n", configfile);
	
	strncpy (parameter, "wprefs_icon", MEDIUM_STRING);
	if (parse_from_config(&uint_param_value, char_param_value, configfile, parameter) == 0){
		strncpy(imagefile1, __CONFPATH, BIG_STRING);
		strcat(imagefile1, "/");	
		strcat(imagefile1, char_param_value);
		//fprintf(stderr, "%s\n", char_param_value);
	}
	strncpy (parameter, "halt_icon", MEDIUM_STRING);
	if (parse_from_config(&uint_param_value, char_param_value, configfile, parameter) == 0){
		strncpy(imagefile2, __CONFPATH, BIG_STRING);	
		strcat(imagefile2, "/");
		strcat(imagefile2, char_param_value);
	}
	strncpy (parameter, "wprefs_cmd", MEDIUM_STRING);
	if (parse_from_config(&uint_param_value, char_param_value, configfile, parameter) == 0){
		strncpy(wprefs_cmd, char_param_value, BIG_STRING);
			strcat (wprefs_cmd, "&");
	}
	
	strncpy (parameter, "text_color", MEDIUM_STRING);
	if (parse_from_config(&uint_param_value, char_param_value, configfile, parameter) == 0){
		strncpy(text_color, char_param_value, BIG_STRING);
	}
	
	strncpy (parameter, "text_size", MEDIUM_STRING);
	if (parse_from_config(&uint_param_value, char_param_value, configfile, parameter) == 0){
		strncpy(text_size, char_param_value, BIG_STRING);
	}

		
	gtk_init(&argc, &argv);
	
	mainwin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_widget_realize(mainwin);
	
	mainbox = create_main_dockapp_icon_window(mainwin, 54, argc, argv);
	
	table = gtk_table_new(2,2, TRUE);

	box1 = gtk_event_box_new ();
	box2 = gtk_event_box_new ();
	clockbox = gtk_vbox_new (FALSE, 0);
	gtk_signal_connect (GTK_OBJECT(box1), "button_press_event", GTK_SIGNAL_FUNC(launchonclick), wprefs_cmd);
	gtk_signal_connect (GTK_OBJECT(box2), "button_press_event", GTK_SIGNAL_FUNC(launchdialog), "Shutdown or Reboot");
	
	watchlabel = gtk_label_new(NULL);
	draw_label(watchlabel);
	separator = gtk_hseparator_new ();	
	
	pixmap1 = (gpointer) gdk_pixmap_create_from_xpm (mainwin->window, &mask,
               	NULL, imagefile1);
        pixmap_widget1 = gtk_pixmap_new((gpointer) pixmap1, mask);
	gtk_container_add(GTK_CONTAINER(box1), pixmap_widget1);

	pixmap2 = (gpointer) gdk_pixmap_create_from_xpm (mainwin->window, &mask,
               	NULL, imagefile2);
        pixmap_widget2 = gtk_pixmap_new((gpointer) pixmap2, mask);
	gtk_container_add(GTK_CONTAINER(box2), pixmap_widget2);
		
	gtk_table_attach_defaults(GTK_TABLE(table), clockbox, 0,2, 0,1);
	gtk_table_attach_defaults(GTK_TABLE(table), box1, 0,1, 1,2);
	gtk_table_attach_defaults(GTK_TABLE(table), box2, 1,2, 1,2);
	
	gtk_container_add (GTK_CONTAINER(mainwin), mainbox);	
	gtk_container_add (GTK_CONTAINER (mainbox), table);
	gtk_container_add (GTK_CONTAINER (clockbox), watchlabel);
	gtk_container_add (GTK_CONTAINER (clockbox), separator);
	
	gtk_widget_show_all(mainwin);
	
	gtk_timeout_add( 1000 , draw_label, watchlabel );
	
	gtk_main ();
	
	return(0);
} // end main


// Function create_main_dockapp_icon_window (c) 2000, Alexey Vyskubov <alexey@pepper.spb.ru>

GtkWidget *create_main_dockapp_icon_window
		(GtkWidget *mw,
		unsigned int s,
		int margc,
		char *margv[]) 
{
	GtkWidget *dockappbox; // This will become icon box 
	Window xmw;
	XWMHints *wm_hints;
	
	xmw = GDK_WINDOW_XWINDOW(mw->window);
	
	dockappbox = gtk_event_box_new();
	gtk_widget_set_usize(dockappbox, s, s);
	gtk_container_add (GTK_CONTAINER (mw), dockappbox);
	gtk_widget_realize(dockappbox);
	
	// Time for game with Xlib 
	wm_hints = XAllocWMHints();
	wm_hints->window_group = xmw;
	wm_hints->icon_window = GDK_WINDOW_XWINDOW(dockappbox->window);
	wm_hints->icon_x = 0;
	wm_hints->icon_y = 0; 
	wm_hints->initial_state = WithdrawnState;
	wm_hints->flags = StateHint |
		IconPositionHint |
		WindowGroupHint |
		IconWindowHint;
	
	XSetWMHints(GDK_DISPLAY(), xmw, wm_hints);
	XSetCommand(GDK_DISPLAY(), xmw, margv, margc);
	
	XFree(wm_hints);
	
	return dockappbox;
} // end create_main_dockapp_icon_window


// the function to execute command which is called when button is clicked ****************************
void launchonclick
		(GtkWidget *event_box, 
		GdkEventButton *event,
		gpointer data)
{
	//fprintf(stderr, "data = %s", (char *) data);  //debug
	int output  = system((char *) data);
	if (output){
		fprintf(stderr, "error launching command in function \"launchonclick\"\n");
	}
	return;
} // end launchonclick



// the function to parse the config file for parameters ****************************
int parse_from_config
		(u_int32_t 
		*uint_value, 
		char *char_value, 
		char *filename, 
		char *param )
{
	int c;
	FILE *file;
	char 	*sep, 
				*buffer, 
				*line, 
				*value,
				*arg1,
				*arg2;
	line = malloc(LONG_STRING); 
	buffer = malloc(MEDIUM_STRING); 
	sep = malloc(MEDIUM_STRING); 
	value = malloc(MEDIUM_STRING);
	arg1 = malloc(64);
	arg2 = malloc(64);
	c = 0;	
	
	if((file=fopen(filename, "rb")) == 0) {
		fprintf (stderr,"Error opening file %s !\n",filename);
		exit(EXIT_FAILURE);
	}
	
	while (fgets (line, LONG_STRING, file) !=NULL) {
		if (strstr (line, "[") !=0) continue;
		if (strstr (line, "#") !=0) continue;
		memset(char_value, 0, BIG_STRING);
		memset(arg1, 0, 64);
		memset(arg2, 0, 64);
		c = sscanf (line, "%s %s %s %s %s", buffer, sep, value, arg1, arg2); 
		if (strncmp (param, buffer, MEDIUM_STRING) == 0) {
			strncat(char_value, value, 127);
			if (*arg1) {
				strcat(char_value, " ");
				strncat(char_value, arg1, 63);
			}
			if (*arg2) {
				strcat(char_value, " ");
				strncat(char_value, arg2, 63);
			}
			*uint_value = atoi(char_value);
			fclose (file);
			return(EXIT_SUCCESS);
		}	
	}
	fclose (file);
	return(EXIT_FAILURE);
} // end parse_from_config

// does what it tells
int usage(void) {
	fprintf(stdout,"\nminicontrol v0.2 : a clock, Wprefs launcher, and shutdown/reboot dialog in a single dockapp (jp.guillemin@free.fr)\n");
	fprintf(stdout, "Usage : minicontrol [switch] [configfile]\n"
					"\t  [-c] choose 1 of the default configfiles\n"
					"\t  [-h] display this help\n"
					"\t  [-p] use this switch to enable custom path to your own configfile\n"
					"( default minicontrol.cf file is located in /usr/share/minicontrol/ )\n");
	return(EXIT_SUCCESS);
} // end usage

// does what it tells
int getclocktime(char *clockstring){
	long clock;
	struct tm *clk;

	time(&clock);
	clk=localtime (&clock);
	sprintf (clockstring, "%02d:%02d", clk->tm_hour, clk->tm_min);
return(EXIT_SUCCESS);
} // end getclocktime


// recalculate time and *watchlabel every second
gint draw_label( gpointer data ){
	while (gtk_events_pending()) gtk_main_iteration();	
	GtkLabel *label = (GtkLabel *)data;
	char *string;
	string = malloc(BIG_STRING);
	getclocktime(string);
		//fprintf(stderr, clocklabel2);
	gtk_label_set_text( label, (char *) string);
	gtk_label_set_justify (GTK_LABEL(label), GTK_JUSTIFY_CENTER);
return TRUE;
}



// dialog popup to choose action Reboot Halt Cancel
void launchdialog
		(GtkWidget *parent, 
		GdkEventButton *event,
		gpointer data)
{
	GtkWidget *dialog;
	GtkWidget *table;
	GtkWidget *button1, *button2, *button3;
   	char	*halt_cmd,
		*reboot_cmd;
	halt_cmd = malloc(BIG_STRING);
	reboot_cmd = malloc(BIG_STRING);
	
	// Here we parse the configfile
	strncpy (parameter, "halt_cmd", MEDIUM_STRING);
	if (parse_from_config(&uint_param_value, char_param_value, configfile, parameter) == 0){
		strncpy(halt_cmd, char_param_value, BIG_STRING);
		strcat (halt_cmd, "&");
	}
	strncpy (parameter, "reboot_cmd", MEDIUM_STRING);
	if (parse_from_config(&uint_param_value, char_param_value, configfile, parameter) == 0){
		strncpy(reboot_cmd, char_param_value, BIG_STRING);
		strcat (reboot_cmd, "&");
	}
	
	dialog = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (dialog), "Really exit?");
	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
	gtk_widget_realize(dialog);
	
	table = gtk_table_new(3,1, FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(table), 5);
	gtk_table_set_col_spacings(GTK_TABLE(table), 5);
	 
	button1 = gtk_button_new_with_label ("Shutdown");
	button2 = gtk_button_new_with_label ("  Reboot  ");
	button3 = gtk_button_new_with_label ("  Cancel  ");
	
	gtk_table_attach_defaults(GTK_TABLE(table), button1, 0,1, 0,1);
	gtk_table_attach_defaults(GTK_TABLE(table), button2, 1,2, 0,1);
	gtk_table_attach_defaults(GTK_TABLE(table), button3, 2,3, 0,1);
	
	gtk_signal_connect (GTK_OBJECT(button1), "button_press_event", GTK_SIGNAL_FUNC(launchonclick), halt_cmd);
	gtk_signal_connect (GTK_OBJECT(button2), "button_press_event", GTK_SIGNAL_FUNC(launchonclick), reboot_cmd);
    	gtk_signal_connect (GTK_OBJECT(button3), "clicked", GTK_SIGNAL_FUNC(exitwidget), dialog);
	
	gtk_container_add (GTK_CONTAINER (dialog), table);
	gtk_widget_show_all (dialog);
   return;


} // end launchdialog

void exitwidget (GtkWidget *wgt, gpointer data){
	gtk_widget_destroy (GTK_WIDGET(data));
	return;
} // end exitwidget


